@@ -1,7 +1,7 @@
 /*
  * Nail - a mail user agent derived from Berkeley Mail.
  *
- * Copyright (c) 2000-2002 Gunnar Ritter, Freiburg i. Br., Germany.
+ * Copyright (c) 2000-2004 Gunnar Ritter, Freiburg i. Br., Germany.
  */
 /*
  * These base64 routines are derived from the metamail-2.7 sources which
@@ -23,7 +23,7 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)base64.c	2.2 (gritter) 9/1/02";
+static char sccsid[] = "@(#)base64.c	2.12 (gritter) 7/20/05";
 #endif
 #endif /* not lint */
 
@@ -38,7 +38,7 @@ static char sccsid[] = "@(#)base64.c	2.2 (gritter) 9/1/02";
 
 static const char b64table[] =
 "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-static const char b64index[] = {
+static const signed char b64index[] = {
 	-1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
 	-1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
 	-1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,62, -1,-1,-1,63,
@@ -51,14 +51,13 @@ static const char b64index[] = {
 
 #define char64(c)  ((c) < 0 ? -1 : b64index[(int)(c)])
 
-static signed char	*ctob64 __P((unsigned char *, int));
+static signed char *ctob64(unsigned char *p, int pad);
 
 /*
  * Convert three characters to base64.
  */
 static signed char *
-ctob64(p, pad)
-unsigned char *p;
+ctob64(unsigned char *p, int pad)
 {
 	static signed char b64[4];
 
@@ -67,7 +66,7 @@ unsigned char *p;
 	if (pad == 2) {
 		b64[2] = b64[3] = '=';
 	} else if (pad == 1) {
-		b64[2] = b64table[((p[1] & 0xF) << 2) | ((p[2] & 0xC0) >> 6)];
+		b64[2] = b64table[((p[1] & 0xF) << 2)];
 		b64[3] = '=';
 	} else {
 		b64[2] = b64table[((p[1] & 0xF) << 2) | ((p[2] & 0xC0) >> 6)];
@@ -76,14 +75,49 @@ unsigned char *p;
 	return b64;
 }
 
+char *
+strtob64(const char *p)
+{
+	return memtob64(p, strlen(p));
+}
+
+char *
+memtob64(const void *vp, size_t isz)
+{
+	char	q[3];
+	const char	*p = vp;
+	signed char	*h;
+	size_t	c = 0;
+	int	i, l = 0, sz = 0, pads;
+	char	*rs = NULL;
+
+	if (isz == 0) {
+		rs = smalloc(1);
+		*rs = '\0';
+		return rs;
+	}
+	do {
+		for (pads = 2, i = 0; i <= 2; i++, pads--) {
+			q[i] = p[c++];
+			if (c == isz)
+				break;
+		}
+		h = ctob64((unsigned char *)q, pads);
+		if (l + 5 >= sz)
+			rs = srealloc(rs, sz = l + 100);
+		for (i = 0; i < 4; i++)
+			rs[l++] = h[i];
+	} while (c < isz);
+	rs[l] = '\0';
+	return rs;
+}
+
 /*
  * Write to a file converting to base64. The input must be aligned
  * e.g. at 972 character bounds.
  */
 size_t
-mime_write_tob64(in, fo, is_header)
-struct str *in;
-FILE *fo;
+mime_write_tob64(struct str *in, FILE *fo, int is_header)
 {
 	char *p, *upper, q[3];
 	signed char *h;
@@ -102,12 +136,12 @@ FILE *fo;
 		fwrite(h, sizeof(char), 4, fo);
 		sz += 4, l += 4;
 		if (l >= 71) {
-			sputc('\n', fo), sz++;
+			putc('\n', fo), sz++;
 			l = 0;
 		}
 	}
 	if (l != 0 && !is_header) {
-		sputc('\n', fo), sz++;
+		putc('\n', fo), sz++;
 	}
 	return sz;
 }
@@ -115,9 +149,8 @@ FILE *fo;
 /*
  * Decode from base64.
  */
-void
-mime_fromb64(in, out, is_text)
-struct str *in, *out;
+void 
+mime_fromb64(struct str *in, struct str *out, int is_text)
 {
 	char *p, *q, *upper;
 	signed char c, d, e, f, g;
@@ -203,18 +236,16 @@ struct str *in, *out;
  * As we have only one buffer, this function is not reentrant.
  */
 void
-mime_fromb64_b(in, out, is_text, f)
-struct str *in, *out;
-FILE *f;
+mime_fromb64_b(struct str *in, struct str *out, int is_text, FILE *f)
 {
 	static signed char b[4];
 	static int n;
-	static FILE *f_b = (FILE*)-1;
+	static FILE *f_b = (FILE *)-1;
 	signed char c;
 	int i;
 	struct str nin;
 
-	nin.s = (char*)smalloc(in->l + n);
+	nin.s = smalloc(in->l + n);
 	if (n != 0 && f_b == f) {
 		for (nin.l = 0; nin.l < n; nin.l++)
 			nin.s[nin.l] = b[nin.l];
