@@ -38,7 +38,7 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)cache.c	1.58 (gritter) 10/2/04";
+static char sccsid[] = "@(#)cache.c	1.60 (gritter) 7/24/05";
 #endif
 #endif /* not lint */
 
@@ -268,7 +268,7 @@ putcache(struct mailbox *mp, struct message *m)
 
 	if ((mp->mb_type != MB_IMAP && mp->mb_type != MB_CACHE) ||
 			m->m_uid == 0 || m->m_time == 0 ||
-			(m->m_flag & (MTOUCH|MFULLYCACHED) == MFULLYCACHED))
+			(m->m_flag & (MTOUCH|MFULLYCACHED)) == MFULLYCACHED)
 		return;
 	if (m->m_have & HAVE_BODY)
 		c = 'B';
@@ -590,7 +590,7 @@ cache_setptr(int transparent)
 	ok = OKAY;
 	if (ok == OKAY) {
 		mb.mb_type = MB_CACHE;
-		mb.mb_perm = MB_DELE;
+		mb.mb_perm = Rflag ? 0 : MB_DELE;
 		if (transparent)
 			transflags(omessage, omsgCount, 1);
 		else
