@@ -33,7 +33,7 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)nss.c	1.37 (gritter) 2/12/05";
+static char sccsid[] = "@(#)nss.c	1.43 (gritter) 7/15/05";
 #endif
 #endif /* not lint */
 
@@ -64,6 +64,8 @@ static sigjmp_buf	nssjmp;
 #include <secerr.h>
 #include <smime.h>
 #include <ciferfam.h>
+#include <xconst.h>
+#include <genname.h>
 #include <private/pprio.h>
 
 #include "extern.h"
@@ -72,6 +74,7 @@ static sigjmp_buf	nssjmp;
 
 static char *password_cb(PK11SlotInfo *slot, PRBool retry, void *arg);
 static SECStatus bad_cert_cb(void *arg, PRFileDesc *fd);
+static enum okay nss_check_host(const char *server, struct sock *sp);
 static const char *bad_cert_str(void);
 static enum okay nss_init(void);
 static void nss_select_method(const char *uhp);
@@ -111,10 +114,91 @@ password_cb(PK11SlotInfo *slot, PRBool retry, void *arg)
 static SECStatus
 bad_cert_cb(void *arg, PRFileDesc *fd)
 {
+	if (PORT_GetError() == SSL_ERROR_BAD_CERT_DOMAIN)
+		/*
+		 * We must not use this result. NSS verifies host names
+		 * according to RFC 2818, but we must verify host names
+		 * according to RFC 2595. The rules are different:
+		 *
+		 * - RFC 2818 says that if both a dNSName and a CN are
+		 *   contained in the peer certificate, only the dNSName
+		 *   is used. RFC 2595 encourages to use both.
+		 *
+		 * - RFC 2818 allows the wildcard '*' in any component
+		 *   of the host name. RFC 2595 allows it only as the
+		 *   "left-most name component".
+		 *
+		 * So ignore it and verify separately.
+		 */
+		return SECSuccess;
 	fprintf(stderr, "Error in certificate: %s.\n", bad_cert_str());
 	return ssl_vrfy_decide() == OKAY ? SECSuccess : SECFailure;
 }
 
+/*
+ * Host name checking according to RFC 2595.
+ */
+static enum okay
+nss_check_host(const char *server, struct sock *sp)
+{
+	CERTCertificate	*cert;
+	char	*cn = NULL;
+	enum okay	ok = STOP;
+	PRArenaPool	*arena;
+	CERTGeneralName	*gn;
+	SECItem	altname;
+	AltNameEncodedContext	ec;
+	int	i;
+	const SEC_ASN1Template	gntempl[] = {
+		{ SEC_ASN1_SEQUENCE_OF, 0, SEC_AnyTemplate }
+	};
+
+	if ((cert = SSL_PeerCertificate(sp->s_prfd)) == NULL) {
+		fprintf(stderr, "no certificate from \"%s\"\n", server);
+		return STOP;
+	}
+	arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
+	if (CERT_FindCertExtension(cert, SEC_OID_X509_SUBJECT_ALT_NAME,
+				&altname) == SECSuccess &&
+			SEC_ASN1DecodeItem(arena, &ec, gntempl,
+				&altname) == SECSuccess &&
+			ec.encodedGenName != NULL) {
+		for (i = 0; ec.encodedGenName[i] != NULL; i++) {
+			gn = CERT_DecodeGeneralName(arena, ec.encodedGenName[i],
+					NULL);
+			if (gn->type == certDNSName) {
+				char	*dn = ac_alloc(gn->name.other.len + 1);
+				memcpy(dn, gn->name.other.data,
+						gn->name.other.len);
+				dn[gn->name.other.len] = '\0';
+				if (verbose)
+					fprintf(stderr,
+						"Comparing DNS name: \"%s\"\n",
+						dn);
+				if (rfc2595_hostname_match(server, dn)
+						== OKAY) {
+					ac_free(dn);
+					goto out;
+				}
+				ac_free(dn);
+			}
+		}
+	}
+	if ((cn = CERT_GetCommonName(&cert->subject)) != NULL) {
+		if (verbose)
+			fprintf(stderr, "Comparing common name: \"%s\"\n", cn);
+		ok = rfc2595_hostname_match(server, cn);
+	}
+	if (ok == STOP)
+		fprintf(stderr, "host certificate does not match \"%s\"\n",
+				server);
+out:	if (cn)
+		PORT_Free(cn);
+	PORT_FreeArena(arena, PR_FALSE);
+	CERT_DestroyCertificate(cert);
+	return ok;
+}
+
 static const char *
 bad_cert_str(void)
 {
@@ -213,6 +297,11 @@ ssl_open(const char *server, struct sock *sp, const char *uhp)
 		return STOP;
 	}
 	sp->s_prfd = fdc;
+	if (nss_check_host(server, sp) != OKAY && ssl_vrfy_decide() != OKAY) {
+		PR_Close(fdc);
+		sp->s_prfd = NULL;
+		return STOP;
+	}
 	sp->s_use_ssl = 1;
 	if (verbose) {
 		char	*cipher, *issuer, *subject;
@@ -256,7 +345,7 @@ nss_gen_err(const char *fmt, ...)
 }
 
 FILE *
-smime_sign(FILE *ip)
+smime_sign(FILE *ip, struct header *headp)
 {
 	NSSCMSMessage	*msg;
 	NSSCMSContentInfo	*content;
@@ -265,10 +354,13 @@ smime_sign(FILE *ip)
 	CERTCertificate	*cert;
 	CERTCertDBHandle	*handle;
 	FILE	*hp, *bp, *sp;
+	char	*addr;
 
 	if (nss_init() != OKAY)
 		return NULL;
-	if ((cert = get_signer_cert(myaddr())) == NULL)
+	if ((addr = myorigin(headp)) == NULL)
+		return NULL;
+	if ((cert = get_signer_cert(addr)) == NULL)
 		return NULL;
 	handle = CERT_GetDefaultCertDB();
 	if ((msg = NSS_CMSMessage_Create(NULL)) == NULL) {
@@ -690,12 +782,11 @@ verify1(struct message *m, int n)
 	int	nlevels, i;
 	int	status = 0;
 	int	foundsender = 0;
-	char	*from;
+	char	*sender;
 
 	if ((m = getsig(m, n, &msg)) == NULL)
 		return 1;
-	if ((from = hfield("from", m)) != NULL)
-		from = skin(from);
+	sender = getsender(m);
 	handle = CERT_GetDefaultCertDB();
 	nlevels = NSS_CMSMessage_ContentLevelCount(msg);
 	for (i = 0; i < nlevels; i++) {
@@ -774,13 +865,13 @@ verify1(struct message *m, int n)
 				svs = NSS_CMSUtil_VerificationStatusToString
 					(vs);
 				addr = CERT_GetCertEmailAddress(&cert->subject);
-				if (from != NULL && addr != NULL &&
-							!asccasecmp(from, addr))
-						foundsender++;
+				if (sender != NULL && addr != NULL &&
+						asccasecmp(sender, addr) == 0)
+					foundsender++;
 				else {
 					addr = CERT_GetFirstEmailAddress(cert);
-					while (from && addr) {
-						if (!asccasecmp(from, addr)) {
+					while (sender && addr) {
+						if (!asccasecmp(sender, addr)) {
 							foundsender++;
 							break;
 						}
@@ -815,10 +906,10 @@ verify1(struct message *m, int n)
 		}
 	}
 	if (foundsender == 0) {
-		if (from) {
+		if (sender) {
 			fprintf(stderr, "Signers of message "
 					"%d do not include the sender <%s>\n",
-				n, from);
+				n, sender);
 			status = -1;
 		} else
 			fprintf(stderr, "Warning: Message %d has no From: "
